#!/bin/bash
# Copyright 2010 - 2019 Michal Žejdl
# parallel launching of collector.sh with some load management

confdir=$(dirname $0)/../../core
config=$confdir/defaults.sh
test -f $config || exit
source $config

# smaller limit leads to heavier mysql load
limit=100

# step should be higher than loop overhead
#ustep=100
# minimum usleep value
#minusleep=$ustep
# how much $delay can be left shifted
#maxusleep=$((1000000 * snmpinterval / 2))

# a new file descriptor to prevent pipe from closing
exec 3>$mysqlwpipe

function do_query
{
	test -z "$1" && return 1
	result=($(mysql -e "$1" $select_opts)) || return 1
}

# change it if the next two selects are changed
columns=9

# sysObjectID must exist - it's used in collector.sh and scripts called by it
# organization = trim(leading '.1.3.6.1.4.1.' from substring_index(sysObjectID, '.', 8))
function get_agents4
{
	do_query "select id, address, sysObjectID, ifNumber, \
dot1dStp, \
dot1dTpFdbEntry, \
hostResources, \
ifXEntry, \
entity \
from snmpagent join snmp4 on id=agent \
where status = 0 \
and sysObjectID != '' \
and collecting = 'n' \
and collected <= unix_timestamp() - $snmpinterval \
group by id order by collected limit $limit \
" || return 1
}

function get_agents6
{
	do_query "select id, address, sysObjectID, ifNumber, \
dot1dStp, \
dot1dTpFdbEntry, \
hostResources, \
ifXEntry, \
entity \
from snmpagent join snmp6 on id=agent \
where status = 0 \
and sysObjectID != '' \
and collecting = 'n' \
and collected <= unix_timestamp() - $snmpinterval \
group by id order by collected limit $limit \
" || return 1
}

# clean up locks
echo "update snmpagent set collecting = 'n';" >&3

# count agents with addresses
#number_of_agents=$(mysql -e "select count(distinct id) from snmpagent left join snmp4 on id=snmp4.agent left join snmp6 on id=snmp6.agent where snmp4.status = 0 and snmp6.status is null or snmp4.status is null and snmp6.status = 0 or snmp4.status = 0 and snmp6.status = 0" $select_opts) || number_of_agents=0
# initial usleep value; time quantum per agent [ms]; + 1 to not divide by zero
#delay=$((1000000 * snmpinterval / (number_of_agents + 1)))

while true
do
	for version in 4 6
	do
  		# stop this golem if "šém" is removed
		test -e $rundir/snmp/$(basename $0 .sh).sem || exit
		if ! get_agents$version
		then
			echo "$(date '+%F %T') get_agents$version () returned error" >&2
			sleep 1
			continue
		fi
		#echo "$(date '+%F %T') get_agents$version () results: ${#result[@]}"
		if test ${#result[@]} -eq 0
		then
			# no results, raise delay
			#if test $delay -lt 100000; then
			#	((delay += ustep))
			#else
			#	test $delay -lt $maxusleep && ((delay <<= 1))
			#fi
			#echo "$(date '+%F %T') usleep $delay"
			#usleep $delay
			sleep 1
			continue
		fi
		if test $((${#result[@]} % columns)) -ne 0
		then
			echo "$(date '+%F %T') odd get_agents$version () results: ${#result[@]}, columns: $columns, limit: $limit" >&2
			sleep 1
			continue
		fi
		rows=$((${#result[@]} / columns))
		#echo "$(date '+%F %T') $(basename $0) get_agents$version () rows: $rows"
		for ((ri = 0; ri < $rows; ri++))
		do
			# wait for lock; named pipe can be slow (it happened)
			if ! mysql -e "update snmpagent set collecting = 'y' where id = ${result[ri * columns]}" $modify_opts
			then
				echo "$(date '+%F %T') mysql update lock failed on agent id ${result[ri * columns]}"
				sleep 1
				continue
			fi
			$coredir/snmp/collector.sh \
"${result[ri * columns]}" \
"$version" \
"${result[ri * columns + 1]}" \
"${result[ri * columns + 2]}" \
"${result[ri * columns + 3]}" \
"${result[ri * columns + 4]}" \
"${result[ri * columns + 5]}" \
"${result[ri * columns + 6]}" \
"${result[ri * columns + 7]}" \
"${result[ri * columns + 8]}" \
&
			#if test $delay -lt 100000; then
			#	if test $delay -gt $minusleep; then
			#		((delay -= ustep))
			#	else
			#		echo "$(date '+%F %T') reached minimum usleep value ($minusleep) - this inefficient software can utilize a faster machine" >&2
			#	fi
			#else
			#	((delay >>= 1))
			#fi
			#echo "$(date '+%F %T') usleep $delay"
			#usleep $delay
		done
		# do not sleep if there may be more agents waiting
		test $rows -eq $limit && continue
		sleep 1
	done
done
