#!/bin/bash
# Copyright 2008 - 2010, 2014, 2015 Michal Žejdl
# pro zadanou MAC adresu otestuje DNS, ICMP, SNMP... a vloží do tabulky checked

confdir=$(dirname $0)/../core
config=$confdir/defaults.sh
test -f $config || exit
. $config

mac=$1

if test $(echo $mac | wc -c) -ne 18; then
  echo čekám MAC adresu 17 byte dlouhou, končím
  exit
fi

# pro zadanou MAC adresu vybereme z arp její poslední IP
if ! ip=$(mysql -e "select ip from arp where mac = '$mac' order by time desc limit 1" $modify_opts -s); then
  echo problém při dotazu na db, končím
  exit
fi

if test -z $ip; then
	echo "$(date +'%F %T') $(basename $0): no IPv4 address found for MAC $mac in the table arp"
	mysql -e "insert checked (mac, lastcheck) values ('$mac', unix_timestamp()) on duplicate key update lastcheck = unix_timestamp()" $modify_opts
	exit
fi

# pro vybranou IP adresu zjistíme z arp její poslední MAC
if ! posledni_mac=$(mysql -e "select mac from arp where ip = '$ip' order by time desc limit 1" $modify_opts -s); then
  echo problém při dotazu na db, končím
  exit
fi

time=$(date +%s)

# jestliže naše MAC není poslední, která použila IP, pak nemůžeme pomocí této IP naši MAC zkoumat
if test $mac != $posledni_mac; then
  # nejdřív jsem chtěl takovou MAC z checked smazat, protože Michalovo dotaz už mi ji znova z arp nevybere
  # až na rychlé flip-flopy na 10.52 - mají stejný čas (Michalovo dotaz mi tu stejnou znova předhodí)
  # takže jen upravíme řádek v tabulce, aby se zmenšila entropie
  # předpokládáme, že ji stačí vypnout (live = n) a info o SNMP můžeme ponechat
  mysql -e "insert checked (mac, lastcheck) values ('$mac', $time) on duplicate key update dns='', lastcheck=$time, live='n'" $modify_opts
  echo IP adresa $ip byla recyklována MAC adresou $posledni_mac, končím
  exit
fi

if host=$(host $ip); then
  dns=$(echo $host | awk '{print $5}' | awk -F. '{print $1}')
else
  dns=""
fi

if ping -c 1 -q $ip > /dev/null; then
  # je živá
  # test na SNMP pomocí sysObjectID.0
  if snmpget -v 1 -c public $ip sysObjectID.0 &> /dev/null; then
    snmp=y
    # sloupec hasfdb existuje proto, že tenhle test trvá dost dlouho na to,
    #   abych jej volal několikrát za vteřinu ze skriptu fdb.sh
    # asep-*, ...	End of MIB
    # tel-*	No Such Instance currently exists at this OID
    if test $(snmpwalk -v 1 -c public $ip mib-2.17.4.3.1.1 2> /dev/null | egrep -v "(^End of MIB$|No Such Instance currently exists at this OID$)" | head -1 | wc -l) -eq 1; then hasfdb=y
    else hasfdb=n
    fi
  else
    snmp=n
    hasfdb=n
  fi
  # insert/update včetně SNMP
  mysql -e "insert checked (mac, dns, lastcheck, live, snmp, hasfdb) values ('$mac', '$dns', $time, 'y', '$snmp', '$hasfdb') on duplicate key update dns = '$dns', lastcheck = $time, live = 'y', snmp = '$snmp', hasfdb = '$hasfdb'" $modify_opts
else
  # není živá
  # insert/update bez SNMP
  mysql -e "insert checked (mac, dns, lastcheck, live) values ('$mac', '$dns', $time, 'n') on duplicate key update dns='$dns', lastcheck=$time, live='n'" $modify_opts
fi
