#include "list.h"

// malloc()
#include <stdlib.h>

int list_insert(struct list_element **head, void *data)
{
	struct list_element *new;

	if ((new = malloc(sizeof(struct list_element)))) {
		// successful malloc()
		new->data = data;
		new->next = *head;
		*head = new;
		return 0;
	} else
		return 1;
}

int list_append(struct list_element *after, void *data)
{
	struct list_element *new;

	// there is no element for appending
	if (! after) return 1;

	if ((new = malloc(sizeof(struct list_element)))) {
		// successful malloc()
		new->data = data;
		new->next = after->next;
		after->next = new;
		return 0;
	} else
		return 1;
}

void list_delete_head(struct list_element **head)
{
	struct list_element *old;

	// if list is non empty
	if (*head) {
		old = *head;
		*head = (*head)->next;
		free(old);
	}
}

void list_delete_after(struct list_element *after)
{
	struct list_element *old;

	// if there is element for deleting
	if (after && after->next) {
		old = after->next;
		after->next = old->next;
		free(old);
	}
}

int list_contains(struct list_element *head, void *data)
{
	while (head) {
		if (head->data == data) return 1;
		head = head->next;
	}

	return 0;
}

int lists_share_data(struct list_element *head_a, struct list_element *head_b)
{
	while (head_a) {
		if (list_contains(head_b, head_a->data)) return 1;
		head_a = head_a->next;
	}

	return 0;
}

struct list_element *list_find_prev(struct list_element *head, void *data)
{
	struct list_element *prev = NULL, *this = head;

	while (this) {
		if (this->data == data) return prev;
		prev = this;
		this = this->next;
	}

	// nothing found
	return NULL;
}

unsigned long list_count_recursive(struct list_element *head)
{
	if (head) return 1 + list_count_recursive(head->next);

	return 0;
}

unsigned long list_count(struct list_element *head)
{
	unsigned long count = 0;

	while (head) {
		count++;
		head = head->next;
	}

	return count;
}

// delete the first occurrence of data
void list_delete_data(struct list_element **head, void *data)
{
	struct list_element *prev, *this;

	// return if list is empty
	if (! *head) return;

	if ((*head)->data == data)
		list_delete_head(head);
	else {
		prev = *head;
		this = (*head)->next;
		while (this) {
			if (this->data == data) {
				list_delete_after(prev);
				return;
			}
			prev = this;
			this = this->next;
		}
	}
}
