#ifndef		LIST_H
#define		LIST_H

/*
 * simple implementation of singly linked list 
 *
 * - similar to SLIST_* macros in sys/queue.h
 * - simplified struct list_element
 *   (contains only pointer to data and pointer to next element)
 * - malloc() and free() for list_element included
 */

struct list_element {
	void *data;
	struct list_element *next;
};

int list_insert(struct list_element **head, void *data);
int list_append(struct list_element *after, void *data);
void list_delete_head(struct list_element **head);
void list_delete_after(struct list_element *after);

// test if list contains given pointer to data
int list_contains(struct list_element *head, void *data);

// test if two lists share any data (intersection)
int lists_share_data(struct list_element *head_a, struct list_element *head_b);

struct list_element *list_find_prev(struct list_element *head, void *data);

/*
 * hitting the stack size with about 256 k of list elements (SIGSEGV)
 * so use recursion with care
 */
unsigned long list_count_recursive(struct list_element *head);

unsigned long list_count(struct list_element *head);

void list_delete_data(struct list_element **head, void *data);

#endif
