#include "output.h"

// struct queue
#include "common.h"
// struct list
#include "list.h"

// printf()
#include <stdio.h>
// inet_ntoa()
#include <arpa/inet.h>
// free()
#include <stdlib.h>

void print_full_mac(const u_char *mac)
{
	int i;
	printf("%02x", mac[0]);
	for (i = 1; i < 6; i++)	printf(":%02x", mac[i]);
}

void print_arp_row(struct arp_row *arp_row)
{
	// insert arp values (\""tolower($1)"\", \""$2"\", "seconds[1]") on duplicate key update time="seconds[1]";
	printf("insert arp values (\"");
	print_full_mac(arp_row->mac);
	// inet_ntoa is deprecated, inet_ntop is preferred
	printf("\", \"%s\", %ld) on duplicate key update time = %ld;\n",
			inet_ntoa(*(struct in_addr *)arp_row->ip4),
			arp_row->time, arp_row->time);
}

void print_arp(struct list *arp)
{
	struct arp_row *arp_row;
	while ((arp_row = list_cut_head(arp)) != NULL) {
		print_arp_row(arp_row);
		free(arp_row);
	}
}

void print_full_ip6(const u_char *ip6)
{
	int i;
	printf("%02x%02x", ip6[0], ip6[1]);
	for (i = 1; i < 8; i++) printf(":%02x%02x", ip6[2 * i], ip6[2 * i + 1]);
}

void print_nd_row(struct nd_row *nd_row)
{
	// insert nd values (\""$1"\", \""$2"\", "$3") on duplicate key update time = "$3";
	printf("insert nd values (\"");
	print_full_mac(nd_row->mac);
	printf("\", \"");
	print_full_ip6(nd_row->ip6);
	printf("\", %ld) on duplicate key update time = %ld;\n",
			nd_row->time, nd_row->time);
}

void print_nd(struct list *nd)
{
	struct nd_row *nd_row;
	while ((nd_row = list_cut_head(nd)) != NULL) {
		print_nd_row(nd_row);
		free(nd_row);
	}
}

void print_queue(struct queue *queue)
{
	//printf("arp: %d, nd: %d\n", queue->arp->count, queue->nd->count);
	print_arp(queue->arp);
	print_nd(queue->nd);
}
